"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sse_1 = require("./sse");
const stream_1 = require("stream");
const defaultOptions = {
    pingInterval: 60000,
    closeEvent: "close"
};
/**
 * Koa Server Side Events middleware wrapper
 *
 * @param {IKoaSSEOptions} [options=defaultOptions]
 * @returns
 */
const middleware = (options = defaultOptions) => {
    const ssePool = [];
    setInterval(() => {
        for (const sse of ssePool) {
            sse.keepAlive();
        }
        console.log(`[${new Date().toISOString()}] [${ssePool.length} clients connected]: SSE heartbeat ping...`);
    }, options.pingInterval);
    /**
     * Returned middleware
     *
     * @param {Context} ctx
     * @param {() => Promise<void>} next
     * @returns
     */
    return (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        if (ctx.res.headersSent) {
            if (!ctx.sse) {
                console.error('[koa-sse]: response headers already sent, unable to create sse stream');
            }
            return yield next();
        }
        const sse = new sse_1.default(ctx, options);
        ssePool.push(sse);
        /**
         * Destroy and release stream resources if connection closes or errors
         *
         * @returns {Promise<void>}
         */
        const close = () => __awaiter(this, void 0, void 0, function* () {
            // Remove sse instance from pool
            ssePool.splice(ssePool.indexOf(sse), 1);
            // Release stream resources
            sse.unpipe();
            sse.destroy();
            // End the response
            ctx.res.end();
            ctx.socket.destroy();
        });
        sse.on("close", close);
        sse.on("error", close);
        ctx.sse = ctx.response.sse = sse;
        yield next();
        if (!ctx.body) {
            // Set response to sse stream if no body
            ctx.body = ctx.sse;
        }
        else if (ctx.body instanceof stream_1.Writable) {
            // Stream body into sse writable stream exists
            ctx.body = ctx.body.pipe(ctx.sse);
        }
        else {
            // Empty existing body response into sse stream
            ctx.sse.send(ctx.body);
            ctx.body = sse;
        }
    });
};
exports.default = middleware;
