/// <reference types="node" />
import { Context } from "koa";
import { IKoaSSEOptions } from "./interfaces/IKoaSSEOptions";
import { IKoaSSEvent } from "./interfaces/IKoaSSEEvent";
import { IKoaSSE } from "./interfaces/IKoaSSEE";
import { Transform, TransformCallback } from "stream";
/**
 * Koa Server Side Events Middleware
 *
 * @class KoaSSE
 * @extends {Transform}
 * @implements {IKoaSSE}
 */
declare class KoaSSE extends Transform implements IKoaSSE {
    private readonly options;
    /**
     * Creates an instance of KoaSSE.
     *
     * @param {Context} ctx
     * @param {IKoaSSEOptions} options
     */
    constructor(ctx: Context, options: IKoaSSEOptions);
    /**
     * Send "heartbeat" comment to keep connection alive
     *
     */
    keepAlive(): void;
    /**
     * Send server side event
     *
     * @param {(IKoaSSEvent | string)} data
     */
    send(data: IKoaSSEvent | string): void;
    /**
     * Send close event source message and end stream
     *
     */
    close(): void;
    /**
     * Transform stream base class method
     *
     * @param {*} data
     * @param {string} _
     * @param {TransformCallback} __
     * @returns {void}
     */
    _transform(data: any, _: string, cb: TransformCallback): void;
}
export default KoaSSE;
